# frozen_string_literal: true

require 'time'

class Post
  attr_reader :title

  def initialize(title:, timestamp:)
    @title = title
    @time = Time.parse(timestamp)
  end

  def to_s(format)
    "#{format_time(format)} #{@title}"
  end

  def to_h
    {
      title: @title,
      timestamp: @time.iso8601
    }
  end

  def today?
    @time.strftime('%F') == Time.now.strftime('%F')
  end

  private

  def format_time(format)
    format ||= '%H:%M'
    @time.strftime(format)
  end
end
